/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors: Joe Kopena <tjkopena@cs.drexel.edu>
 *
 * This program conducts a simple experiment: It places two nodes at a
 * parameterized distance apart.  One node generates packets and the
 * other node receives.  The stat framework collects data on packet
 * loss.  Outside of this program, a control script uses that data to
 * produce graphs presenting performance at the varying distances.
 * This isn't a typical simulation but is a common "experiment"
 * performed in real life and serves as an accessible exemplar for the
 * stat framework.  It also gives some intuition on the behavior and
 * basic reasonability of the NS-3 WiFi models.
 *
 * Applications used by this program are in test02-apps.h and
 * test02-apps.cc, which should be in the same place as this file.
 * 
 */
#include <ctime>

#include <sstream>

#include "ns3/core-module.h"
#include "ns3/network-module.h"
#include "ns3/mobility-module.h"
#include "ns3/wifi-module.h"
#include "ns3/internet-module.h"
#include "ns3/stats-module.h"

#include "app.h"

using namespace ns3;
using namespace std;

NS_LOG_COMPONENT_DEFINE ("MySim");

void
SenderTx (Ptr<CounterCalculator<uint32_t> > datac,
	  std::string path, Ptr<const Packet> packet)
{
  WifiMacHeader hdr;
  packet->PeekHeader (hdr);
  datac->Update ();
  NS_LOG_INFO (path << " " << Simulator::Now() << hdr);
}

void
ReceiverRx (Ptr<CounterCalculator<uint32_t> > datac,
	    std::string path, Ptr<const Packet> packet)
{
  WifiMacHeader hdr;
  packet->PeekHeader (hdr);
  datac->Update ();
  NS_LOG_INFO (path << " " << Simulator::Now() << hdr);
}

//----------------------------------------------
//-- main
//----------------------------------------------
int main (int argc, char *argv[]) {

  string format ("db");
  string experiment ("distance-test");
  string strategy ("wifi");
  string input;
  string runID;
  double distance = 90;

  {
    stringstream sstr;
    sstr << "run-" << time (NULL);
    runID = sstr.str ();
  }

  // Set up command line parameters used to control the experiment.
  CommandLine cmd;
  cmd.AddValue ("distance", "Distance apart to place nodes (in meters).", distance);
  cmd.AddValue ("format", "Format to use for data output.", format);
  cmd.AddValue ("experiment", "Identifier for experiment.", experiment);
  cmd.AddValue ("strategy", "Identifier for strategy.", strategy);
  cmd.AddValue ("run", "Identifier for run.", runID);
  cmd.Parse (argc, argv);

  #ifndef STATS_HAS_SQLITE3
  if (format == "db") {
      NS_LOG_ERROR ("sqlite support not compiled in.");
      return -1;
    }
  #endif

  {
    stringstream sstr ("");
    sstr << distance;
    input = sstr.str ();
  }

  //------------------------------------------------------------
  //-- Create nodes and network stacks
  //--------------------------------------------
  NS_LOG_INFO ("Creating nodes.");
  NodeContainer nodes;
  nodes.Create (2);

  /* Wifiڑ */
  NS_LOG_INFO ("Installing WiFi and Internet stack.");
  WifiHelper wifi = WifiHelper::Default ();
  NqosWifiMacHelper wifiMac = NqosWifiMacHelper::Default ();
  wifiMac.SetType ("ns3::AdhocWifiMac");
  YansWifiPhyHelper wifiPhy = YansWifiPhyHelper::Default ();
  YansWifiChannelHelper wifiChannel = YansWifiChannelHelper::Default ();
  wifiPhy.SetChannel (wifiChannel.Create ());
  NetDeviceContainer nodeDevices = wifi.Install (wifiPhy, wifiMac, nodes);

  /* lbg[NEgX|[gwڑ */
  InternetStackHelper internet;
  internet.Install (nodes);
  Ipv4AddressHelper ipAddrs;
  ipAddrs.SetBase ("192.168.0.0", "255.255.255.0");
  ipAddrs.Assign (nodeDevices);

  /* reB[ڑ */
  MobilityHelper mobility;
  Ptr<ListPositionAllocator> positionAlloc = CreateObject<ListPositionAllocator>();
  positionAlloc->Add (Vector (0.0, 0.0, 0.0));
  positionAlloc->Add (Vector (0.0, distance, 0.0));
  mobility.SetPositionAllocator (positionAlloc);
  mobility.Install (nodes);

  /* AvP[Vڑ */
  Ptr<Node> source = NodeList::GetNode (0);
  Ptr<Node> sink   = NodeList::GetNode (1);
  Ptr<Sender> sender     = CreateObject<Sender>();
  Ptr<Receiver> receiver = CreateObject<Receiver>();
  source ->AddApplication (sender);
  sink   ->AddApplication (receiver);
  sender  ->SetStartTime (Seconds (0));
  receiver->SetStartTime (Seconds (0));
  Config::Set ("/NodeList/0/ApplicationList/*/$Sender/Destination", Ipv4AddressValue ("192.168.0.2"));

  /* DataRNV */
  DataCollector data;
  data.DescribeRun (experiment, strategy, input, runID);
  data.AddMetadata ("author", "sugiyama");

  /* M񐔃JEg*/
  Ptr<CounterCalculator<uint32_t> > txCounter = CreateObject<CounterCalculator<uint32_t> >();
  txCounter->SetKey ("tx");
  txCounter->SetContext ("node[0]");
  Config::Connect ("/NodeList/0/DeviceList/*/$ns3::WifiNetDevice/Phy/PhyTxBegin",
		   MakeBoundCallback (&SenderTx, txCounter));
  data.AddDataCalculator (txCounter);
  
  /* M񐔃JEg */
  Ptr<CounterCalculator<uint32_t> > rxCounter = CreateObject<CounterCalculator<uint32_t> >();
  rxCounter->SetKey ("rx");
  rxCounter->SetContext ("node[1]");
  Config::Connect ("/NodeList/1/DeviceList/*/$ns3::WifiNetDevice/Phy/PhyRxEnd",
                   MakeBoundCallback (&ReceiverRx, rxCounter));
  data.AddDataCalculator (rxCounter);

  /* V~[V̎s */
  Simulator::Run ();

  /* SQLiteŏݏ*/
  Ptr<DataOutputInterface> output = 0;
  output = CreateObject<SqliteDataOutput>();
  output->Output (data);

  /* V~[V̏I */
  Simulator::Destroy ();
}
